# include	"CIcon2GIFApplication.h"
# include	"gd.h"

void main( )
{
	CIcon2GIFApplication* theApp = new CIcon2GIFApplication;
	theApp->Run( );
	delete theApp;
}

CIcon2GIFApplication::CIcon2GIFApplication( )
	: BApplication( 'I2GF' )
{
	mReceivedCommand = FALSE;
}

CIcon2GIFApplication::~CIcon2GIFApplication( )
{

}

void CIcon2GIFApplication::MessageReceived( BMessage* inMessage )
{
	switch ( inMessage->what )
	{
		default:
			BApplication::MessageReceived( inMessage );
	}
}

void CIcon2GIFApplication::AboutRequested( )
{
	BAlert* about = new BAlert( "", "Icon2GIF 1.0b2\nCopyright 1996 Eric Kidd\n\n" \
		"<http://www.pobox.com/~emk/>", "OK" );
	about->Go( );
}

void CIcon2GIFApplication::RefsReceived( BMessage* inMessage )
{
	ulong type;
	long count;

	inMessage->GetInfo( "refs", &type, &count );
	for ( int i = 0; i < count; i++ )
	{
		record_ref item = inMessage->FindRef( "refs", i );
		if ( item.record >= 0 && item.database >= 0 )
		{
			if ( does_ref_conform( item, "BrowserItem" ) )
			{
				BRecord* rec = new BRecord( item );
				if ( rec->Error( ) == B_NO_ERROR )
					ProcessBrowserItem( rec );
				delete rec;
			}
		}
	}
	mReceivedCommand = TRUE;
}

void CIcon2GIFApplication::ReadyToRun( )
{
	if ( mReceivedCommand )
		PostMessage( B_QUIT_REQUESTED );
	else
		RunFilePanel( "Icon2GIF : Create GIF of Icon" );
}

void CIcon2GIFApplication::FilePanelClosed( BMessage* inMessage )
{
	PostMessage( B_QUIT_REQUESTED );
}

void CIcon2GIFApplication::ProcessBrowserItem( BRecord* inItem )
{
	record_id iconID = inItem->FindRecordID( "iconRef" );
	if ( iconID == 0 )
	{
			// Cope with any errors
		BAlert* alert = new BAlert( "", "This icon is hiding!", "OK" );
		alert->Go( );
		return;
	}
	
	BRecord* iconRecord = new BRecord( inItem->Database( ), iconID );
	if ( iconRecord->Error( ) != B_NO_ERROR )
	{
			// Cope with any errors
		BAlert* alert = new BAlert( "", "Couldn't get icon record!", "OK" );
		alert->Go( );
		delete iconRecord;
		return;
	}
	
	long bitsLength;
	uchar* iconBits = (uchar*) iconRecord->FindRaw( "largeBits", &bitsLength );
	
	gdImagePtr im = gdImageCreate( 32, 32 );

	color_map* beColors = system_colors( );

	bool usedColors[256];
	for ( int i = 0; i < 256; i++ )
		usedColors[i] = FALSE;
	for ( int i = 0; i < bitsLength; i++ )
		usedColors[iconBits[i]] = TRUE;

	int gifColors[256];
	for ( int i = 0; i < 256; i++ )
	{
		if ( usedColors[i] == TRUE )
			gifColors[i] = gdImageColorAllocate( im,
				beColors->color_list[i].red,
				beColors->color_list[i].green,
				beColors->color_list[i].blue );
	}

	gdImageInterlace( im, 1 );
	if ( usedColors[B_TRANSPARENT_8_BIT] == TRUE )
		gdImageColorTransparent( im, gifColors[B_TRANSPARENT_8_BIT] );

	for ( int x = 0; x < 32; x++ )
		for ( int y = 0; y < 32; y++ )
			gdImageSetPixel( im, x, y, gifColors[iconBits[x + 32*y]] );

	string pathname = RecordToGIFPath( inItem );
	FILE* out = fopen( pathname, "wb" );
	gdImageGif( im, out );
	fclose( out );	

	gdImageDestroy( im );
	delete iconRecord;
	
		// Set the icon to a BeShow GIF
	record_ref gifRef;
	if ( get_ref_for_path( pathname, &gifRef ) == B_NO_ERROR )
	{
		BFile* gifFile = new BFile( gifRef );
		if ( gifFile && !gifFile->Error( ) )
			gifFile->SetTypeAndApp( 'GIFf', 'iMgV' );
		if ( gifFile )
			delete gifFile;
	}
}

string CIcon2GIFApplication::RecordToGIFPath( BRecord* inRecord )
{
	string	newPath = "icon";
#	define	MAX_PATH_LEN	(4096)
	char	pathBuffer[MAX_PATH_LEN]; // XXX - Should be unlimited
	
	if ( does_ref_conform( inRecord->Ref( ), "File" ) )
	{
		BFile file( inRecord->Ref( ) );
		file.GetPath( pathBuffer, MAX_PATH_LEN ); // XXX - Check for err 
		newPath = pathBuffer;
	}
	// XXX - Argh! Queries are a subclass of folder?
//	else if ( does_ref_conform( inRecord->Ref( ), "Folder" ) )
//	{
//		BDirectory directory( inRecord->Ref( ) );
//		directory.GetPath( pathBuffer, MAX_PATH_LEN ); // XXX - Check for err 
//		newPath = pathBuffer;
//	}
	else
	{
			// XXX - Put pathless items at top level
		newPath = inRecord->FindString( "Name" );
	}
	
	newPath += ".gif";
	return newPath;
}